﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;
using System.Windows.Threading;
using Telerik.Windows.Controls.Map;
using System.Collections.ObjectModel;

namespace OSMultimodeProvider
{
	// The sample of custom map provider that supports multiple map mode.
	public class CustomOpenStreetMapProvider : MapProviderBase
	{
		/// <summary>
		/// Initializes a new instance of the CustomOpenStreetMapProvider class.
		/// </summary>
		/// <param name="mode">Map mode.</param>
		/// <param name="labelVisible">Is labels visible.</param>
		public CustomOpenStreetMapProvider(MapMode mode, bool labelVisible)
			: base(mode, labelVisible)
		{
		}

		/// <summary>
		/// Initializes a new instance of the CustomOpenStreetMapProvider class.
		/// </summary>
		public CustomOpenStreetMapProvider()
			: this(MapMode.Road, true)
		{
		}

		/// <summary>
		/// Gets value which indicates whether labels are supported by the map provider.
		/// </summary>
		public override bool IsLabelSupported
		{
			get
			{
				return false;
			}
		}

		/// <summary>
		/// Returns the SpatialReference for the map provider.
		/// </summary>
		public override ISpatialReference SpatialReference
		{
			get
			{
				return new MercatorProjection();
			}
		}

		/// <summary>
		/// Gets list of the supported map modes.
		/// </summary>
		/// <returns>List of the supported map modes.</returns>
		public override IEnumerable<MapMode> SupportedModes
		{
			get
			{
				yield return MapMode.Road;
				yield return MapMode.Aerial;
			}
		}

		/// <summary>
		/// Initialize provider.
		/// </summary>
		public override void Initialize()
		{
		}

		/// <summary>
		/// Gets value which indicates whether given mode is supported by map provider.
		/// </summary>
		/// <param name="mode">Map mode to check.</param>
		/// <returns>true if given mode is supported. Otherwise - false.</returns>
		public override bool IsModeSupported(MapMode mode)
		{
			return true;
		}

		/// <summary>
		/// MapModeChanged handler.
		/// </summary>
		protected override void OnMapModeChanged(MapMode oldMode, MapMode newMode)
		{
		}

		/// <summary>
		/// Returns map source for required map mode for supporting multiple map mode.
		/// </summary>
		/// <param name="mode">Map mode.</param>
		/// <param name="isLabelVisible">Is label visible.</param>
		/// <returns>Map source.</returns>
		public override MapProviderBase GetSource(MapMode mode, bool isLabelVisible)
		{
			string modeId = string.Format("{0}_{1}", mode, isLabelVisible);
			CustomOpenStreetMapSource source = null;
			if (this.SourceList.ContainsKey(modeId))
			{
				source = this.SourceList[modeId] as CustomOpenStreetMapSource;
			}
			else
			{
				switch (this.Mode)
				{
					case MapMode.Road:
						source = new CustomOpenStreetMapnikSource();
						break;

					case MapMode.Aerial:
						source = new CustomOpenStreetOsmarenderSource();
						break;
				}

				this.SourceList[modeId] = source;
			}

			return source;
		}
	}
}
