﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.Map;
using System.Collections.Generic;
using System.Globalization;

namespace OSMultimodeProvider
{
	public abstract class CustomOpenStreetMapSource : MapProviderBase
	{
		private const string TileMapnikUrlFormat = @"http://a.tile.openstreetmap.org/{zoom}/{x}/{y}.png";
		private const string TileOsmarenderUrlFormat = @"http://a.tah.openstreetmap.org/Tiles/tile/{zoom}/{x}/{y}.png";

		private bool initialized;

		/// <summary>
		/// Initializes a new instance of the CustomOpenStreetMapSource class.
		/// </summary>
		/// <param name="mode">Map mode.</param>
		/// <param name="labelVisible">Is labels visible.</param>
		public CustomOpenStreetMapSource(MapMode mode, bool labelVisible)
			: base(mode, labelVisible)
		{
		}

		/// <summary>
		/// Gets the IsInitialized property.
		/// Indicates that the provider is initialized.
		/// </summary>
		public override bool IsInitialized
		{
			get
			{
				return this.initialized;
			}
		}

		public override void Initialize()
		{
			this.initialized = base.IsInitialized;
		}

		/// <summary>
		/// Gets value which indicates whether labels are supported by the map provider.
		/// </summary>
		public override bool IsLabelSupported
		{
			get
			{
				return false;
			}
		}

		/// <summary>
		/// Gets list of the supported map modes.
		/// </summary>
		/// <returns>List of the supported map modes.</returns>
		public override IEnumerable<MapMode> SupportedModes
		{
			get
			{
				yield return MapMode.Road;
				yield return MapMode.Aerial;
			}
		}

		/// <summary>
		/// Gets value which indicates whether given mode is supported by map provider.
		/// </summary>
		/// <param name="mode">Map mode to check.</param>
		/// <returns>true if given mode is supported. Otherwise - false.</returns>
		public override bool IsModeSupported(MapMode mode)
		{
			return true;
		}

		public override Uri GetTile(int tileLevel, int tilePositionX, int tilePositionY)
		{
			int zoomLevel = this.ConvertTileToZoomLevel(tileLevel);

			string url = string.Empty;

			switch (this.Mode)
			{
				case MapMode.Road:
					url = TileMapnikUrlFormat;
					break;

				case MapMode.Aerial:
					url = TileOsmarenderUrlFormat;
					break;
			}

			url = ProtocolHelper.SetScheme(url);
			url = url.Replace("{zoom}", zoomLevel.ToString(CultureInfo.InvariantCulture));
			url = url.Replace("{x}", tilePositionX.ToString(CultureInfo.InvariantCulture));
			url = url.Replace("{y}", tilePositionY.ToString(CultureInfo.InvariantCulture));

			return new Uri(url);
		}

		protected override void OnMapModeChanged(MapMode oldMode, MapMode newMode)
		{
		}
	}
}
